import java.util.Scanner;

public class ConnectFour {

    // Constants for the board size and the disk colors
    public static final int ROWS = 6;
    public static final int COLUMNS = 7;
    public static final char RED = 'R';
    public static final char YELLOW = 'Y';
    public static final char EMPTY = ' ';

    // A 2D array to store the board state
    public static char[][] board = new char[ROWS][COLUMNS];

    // A scanner to read the user input
    public static Scanner input = new Scanner(System.in);

    public static void main(String[] args) {
        // Initialize the board with empty spaces
        for (int i = 0; i < ROWS; i++) {
            for (int j = 0; j < COLUMNS; j++) {
                board[i][j] = EMPTY;
            }
        }

        // A boolean flag to indicate the game status
        boolean gameOver = false;

        // A char variable to store the current player's color
        char currentPlayer = RED;

        // A loop to repeat until the game is over
        while (!gameOver) {
            // Display the board
            displayBoard();

            // Prompt the current player to choose a column
            System.out.print("Please choose a column (0-6) to drop the " + 
                             (currentPlayer == RED ? "RED" : "YELLOW") + " disk: "); // Changed to display full words

            // A variable to store the chosen column
            int column = -1;

            // A boolean flag to indicate the validity of the input
            boolean validInput = false;

            // A loop to repeat until a valid input is entered
            while (!validInput) {
                // Try to read an integer from the input
                try {
                    column = input.nextInt();

                    // Check if the column is within the range and not full
                    if (column >= 0 && column < COLUMNS && board[0][column] == EMPTY) {
                        // Set the input as valid
                        validInput = true;
                    } else {
                        // Display an error message
                        System.out.println("Invalid column. Please try again.");
                    }
                } catch (Exception e) {
                    // Display an error message
                    System.out.println("Invalid input. Please enter an integer.");
                    // Clear the input buffer
                    input.nextLine();
                }
            }

            // Drop the disk in the chosen column
            dropDisk(currentPlayer, column);

            // Check if the current player has won the game
            if (isWinningMove(currentPlayer, column)) {
                // Display the board
                displayBoard();
                // Display the winning message
                System.out.println("Congratulations! " + (currentPlayer == RED ? "RED" : "YELLOW") + " has won the game!"); // Changed to display full words
                // Set the game as over
                gameOver = true;
            } else {
                // Check if the board is full
                if (isBoardFull()) {
                    // Display the board
                    displayBoard();
                    // Display the draw message
                    System.out.println("The game is a draw!");
                    // Set the game as over
                    gameOver = true;
                } else {
                    // Switch to the next player
                    currentPlayer = (currentPlayer == RED) ? YELLOW : RED;
                }
            }
        }
    }

    // A method to display the board on the console
    public static void displayBoard() {
        // Print the column numbers
        for (int i = 0; i < COLUMNS; i++) {
            System.out.print(i + " ");
        }
        System.out.println();

        // Print the board contents
        for (int i = 0; i < ROWS; i++) {
            System.out.print("|");
            for (int j = 0; j < COLUMNS; j++) {
                System.out.print(board[i][j] + "|");
            }
            System.out.println();
        }
    }

    // A method to drop a disk in a column
    public static void dropDisk(char color, int column) {
        // Find the lowest empty row in the column
        int row = ROWS - 1;
        while (board[row][column] != EMPTY) {
            row--;
        }

        // Place the disk in the row
        board[row][column] = color;
    }

    // A method to check if a disk in a column is a winning move
    public static boolean isWinningMove(char color, int column) {
        // Find the row of the disk
        int row = ROWS - 1;
        while (board[row][column] != color) {
            row--;
        }

        // Check for horizontal win
        int count = 0;
        for (int j = 0; j < COLUMNS; j++) {
            if (board[row][j] == color) {
                count++;
                if (count == 4) {
                    return true;
                }
            } else {
                count = 0;
            }
        }

        // Check for vertical win
        count = 0;
        for (int i = 0; i < ROWS; i++) {
            if (board[i][column] == color) {
                count++;
                if (count == 4) {
                    return true;
                }
            } else {
                count = 0;
            }
        }

        // Check for diagonal win (top-left to bottom-right)
        count = 0;
        int i = row - column;
        int j = 0;
        if (i < 0) {
            j = -i;
            i = 0;
        }
        while (i < ROWS && j < COLUMNS) {
            if (board[i][j] == color) {
                count++;
                if (count == 4) {
                    return true;
                }
            } else {
                count = 0;
            }
            i++;
            j++;
        }

        // Check for diagonal win (bottom-left to top-right)
        count = 0;
        i = row + column;
        j = 0;
        if (i > ROWS - 1) {
            j = i - (ROWS - 1);
            i = ROWS - 1;
        }
        while (i >= 0 && j < COLUMNS) {
            if (board[i][j] == color) {
                count++;
                if (count == 4) {
                    return true;
                }
            } else {
                count = 0;
            }
            i--;
            j++;
        }

        // If none of the above conditions are met, return false
        return false;
    }

    // A method to check if the board is full
    public static boolean isBoardFull() {
        // Check if the top row is full
        for (int j = 0; j < COLUMNS; j++) {
            if (board[0][j] == EMPTY) {
               return false;
            }
        }
        // If the top row is full, return true
    return true;
    }
}
